# This file was auto-generated by Fern from our API Definition.

import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from ...core.api_error import ApiError
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_headers import remove_none_from_headers
from ...environment import CandidApiEnvironment
from ..commons.types.encounter_id import EncounterId
from .types.billing_note import BillingNote


class BillingNotesClient:
    def __init__(self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: str):
        self._environment = environment
        self._token = token

    def create(self, *, encounter_id: EncounterId, text: str) -> BillingNote:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "api/billing_notes/v2"),
            json=jsonable_encoder({"encounter_id": encounter_id, "text": text}),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BillingNote, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBillingNotesClient:
    def __init__(self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: str):
        self._environment = environment
        self._token = token

    async def create(self, *, encounter_id: EncounterId, text: str) -> BillingNote:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "api/billing_notes/v2"),
                json=jsonable_encoder({"encounter_id": encounter_id, "text": text}),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BillingNote, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...environment import CandidApiEnvironment
from .resources.v_3.client import AsyncV3Client, V3Client


class EncountersClient:
    def __init__(self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: str):
        self._environment = environment
        self._token = token
        self.v_3 = V3Client(environment=self._environment, token=self._token)


class AsyncEncountersClient:
    def __init__(self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: str):
        self._environment = environment
        self._token = token
        self.v_3 = AsyncV3Client(environment=self._environment, token=self._token)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime


class EncounterProviderBase(pydantic.BaseModel):
    first_name: typing.Optional[str] = pydantic.Field(
        description=("If the billing provider is an individual, this should be set instead of organization name\n")
    )
    last_name: typing.Optional[str] = pydantic.Field(
        description=("If the billing provider is an individual, this should be set instead of organization name\n")
    )
    organization_name: typing.Optional[str] = pydantic.Field(
        description=("If the billing provider is an organization, this should be set instead of first + last name\n")
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class InvoiceStatus(str, enum.Enum):
    DRAFT = "draft"
    OPEN = "open"
    PAID = "paid"
    VOID = "void"
    UNCOLLECTIBLE = "uncollectible"

    def visit(
        self,
        draft: typing.Callable[[], T_Result],
        open: typing.Callable[[], T_Result],
        paid: typing.Callable[[], T_Result],
        void: typing.Callable[[], T_Result],
        uncollectible: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is InvoiceStatus.DRAFT:
            return draft()
        if self is InvoiceStatus.OPEN:
            return open()
        if self is InvoiceStatus.PAID:
            return paid()
        if self is InvoiceStatus.VOID:
            return void()
        if self is InvoiceStatus.UNCOLLECTIBLE:
            return uncollectible()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import uuid

import pydantic

from ....core.datetime_utils import serialize_datetime


class Payer(pydantic.BaseModel):
    payer_uuid: typing.Optional[uuid.UUID] = pydantic.Field(description=("Auto-generated ID set on creation\n"))
    payer_id: str = pydantic.Field(description=("The primary national payer ID of the payer\n"))
    payer_name: str = pydantic.Field(description=("The primary display name of the payer\n"))

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .expected_network_status import ExpectedNetworkStatus
from .expected_network_status_response import ExpectedNetworkStatusResponse

__all__ = ["ExpectedNetworkStatus", "ExpectedNetworkStatusResponse"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ...commons.types.claim_id import ClaimId
from ...era.types.era import Era
from ...service_lines.types.service_line import ServiceLine
from .claim_status import ClaimStatus


class Claim(pydantic.BaseModel):
    claim_id: ClaimId
    status: ClaimStatus
    clearinghouse: typing.Optional[str]
    clearinghouse_claim_id: typing.Optional[str]
    payer_claim_id: typing.Optional[str]
    service_lines: typing.List[ServiceLine]
    eras: typing.List[Era]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}
